float4x4 mtx_world;
float4x4 mtx_world_inv;
float4x4 mtx_proj;

float fog_exp = 0.3;

struct VS_OUTPUT
{
	float4 pos : POSITION;
	float4 color : COLOR0;
	float3 worldpos : TEXCOORD0;
	float3 viewpos : TEXCOORD1;
};

VS_OUTPUT BasicVS(
	float4 vpos : POSITION,
	float3 vnormal : NORMAL,
	float4 vcolor : COLOR0)
{
	VS_OUTPUT ret;

	// Calculate the view position and projected position	
	float4 wpos = mul(vpos, mtx_world);
	ret.pos = mul(wpos, mtx_proj);
	
	// Perturbate the world normal
	vnormal.x *= 0.3f + (1.f + sin(vpos.x * 0.1111f) * cos(vpos.y * 0.23232f)) * 0.5f;
	vnormal.y *= 0.3f + (1.f + sin(vpos.y * 0.2222f) * cos(vpos.z * 0.33232f)) * 0.5f;
	vnormal.z *= 0.3f + (1.f + sin(vpos.z * 0.3333f) * cos(vpos.x * 0.13232f)) * 0.5f;
	
	// Calculate the lighting
	float3 normal = -mul(normalize(vnormal), (float3x3)mtx_world);
//	float light_amt = dot(normal, float3(0, 0, 1));
	float light_amt = abs(dot(normal, normalize(wpos)));
	
	// Calculate the final color
	ret.color = vcolor * light_amt;
/*	ret.color = float4((float3(1.f, 1.f, 1.f) + normal) * 0.5f, 1.f);
	ret.color.x = 0;
	ret.color.y = 0;*/
//	ret.color = float4(normal.z, normal.z, normal.z, 1.f);
	
	// Return the world position
	ret.worldpos = vpos;
	ret.viewpos = wpos;
	return ret;
}

float4 BasicPS(VS_OUTPUT In) : COLOR0
{
	float4 col = In.color;
	
	float m = 1.f + sin(In.worldpos.x * 0.33 + In.worldpos.z * 0.1662f) * 0.1f;
	m *= 1.f + sin(In.worldpos.y * 0.44 + In.worldpos.x * 0.662f) * 0.1f;
	m *= 1.f + sin(In.worldpos.z * 0.55 + In.worldpos.y * 0.41662f) * 0.1f;
	col *= m;
	
	// Apply a distance based fogging
	col = lerp(col, float4(0.35f, 0.2f, 0.1f, 1.0f), clamp(pow(In.viewpos.z * 0.02f, fog_exp), 0.f, 1.f));
	return col;
}

technique Basic
{
	pass P0
	{
		VertexShader = compile vs_2_0 BasicVS();
		PixelShader = compile ps_2_0 BasicPS();
	}
}
