#define _CRT_SECURE_NO_WARNINGS
#include "Error.h"
#include "Math_Util.h"
#include "Sound.h"

#include <stdio.h>
#include <stdlib.h>

Sound::Sound()
	: m_system(0)
	, m_sound_timer(1.f)
{
	FMOD_RESULT res;

	// Init all pointers to null
	for (int i = 0; i < Fx_COUNT; ++i)
	{
		m_fx_sounds[i] = 0;
	}
	for (int i = 0; i < Ambient_File_Count; ++i)
	{
		m_ambient_sounds[i] = 0;
	}

	// Create the FMod system
	res = FMOD::System_Create(&m_system);
	if (res != FMOD_OK)
		throw Error("Error creating the main FMod audio system!");

	// Initialize the sound
	res = m_system->init(100, FMOD_INIT_NORMAL, 0);
	if (res != FMOD_OK)
		throw Error("Error initializing audio!");

	// Load the audio files
	m_system->createSound("start.wav", FMOD_DEFAULT, 0, &m_fx_sounds[Fx_Start]);
	m_system->createSound("die.wav", FMOD_DEFAULT, 0, &m_fx_sounds[Fx_Die]);

	for (int i = 0; i < Ambient_File_Count; ++i)
	{
		char fname[32];
		sprintf(fname, "%d.wav", i + 1);

		m_system->createSound(fname, FMOD_DEFAULT, 0, &m_ambient_sounds[i]);
	}
}

Sound::~Sound()
{
}

void Sound::Update(float delta_t)
{
	// Update FMod
	m_system->update();

	// Play random sounds
	m_sound_timer -= delta_t;
	if (m_sound_timer < 0.f)
	{
		m_sound_timer = Rand_Float(0.5f, 1.6f);

		// Pick a random sound
		int sound_idx = rand() % Ambient_File_Count;
		if (m_ambient_sounds[sound_idx] == 0)
			return;

		// Start the sound
		FMOD::Channel *channel = 0;
		m_system->playSound(FMOD_CHANNEL_FREE, m_ambient_sounds[sound_idx], true, &channel);
		if (channel != 0)
		{
			float freq = 0.f;
			channel->getFrequency(&freq);

			channel->setFrequency(freq * Rand_Float(0.1f, 1.2f));
			channel->setVolume(Rand_Float(0.1f, 0.3f));
			channel->setPaused(false);
		}
	}
}

void Sound::Play(Fx_Type fx)
{
	if (fx < 0 || fx >= Fx_COUNT)
		return;
	if (m_fx_sounds[fx] == 0)
		return;

	FMOD::Channel *channel = 0;
	m_system->playSound(FMOD_CHANNEL_FREE, m_fx_sounds[fx], false, &channel);
	if (channel == 0)
		return;
}
